@@ -22,7 +22,6 @@
 
 #include <glib/gi18n.h>
 
-#include <stdlib.h>
 #include <sys/utsname.h>
 #include <sys/wait.h>
 
@@ -50,10 +49,10 @@ struct _PtyxisApplication
   PtyxisShortcuts     *shortcuts;
   PtyxisContainerMenu *container_menu;
   PtyxisProfileMenu   *profile_menu;
+  char                *next_title_prefix;
   char                *system_font_name;
   GDBusProxy          *portal;
   PtyxisClient        *client;
-  GHashTable          *exited;
   GVariant            *session;
   guint                has_restored_session : 1;
   guint                overlay_scrollbars : 1;
@@ -81,6 +80,8 @@ static void ptyxis_application_preferences       (GSimpleAction *action,
 static void ptyxis_application_focus_tab_by_uuid (GSimpleAction *action,
                                                   GVariant      *param,
                                                   gpointer       user_data);
+static void ptyxis_application_apply_default_size(PtyxisApplication *self,
+                                                  PtyxisTerminal    *terminal);
 
 static GActionEntry action_entries[] = {
   { "about", ptyxis_application_about },
@@ -148,6 +149,63 @@ ptyxis_application_restore (PtyxisApplication *self)
   return ptyxis_session_restore (self, self->session);
 }
 
+static void
+ptyxis_application_open (GApplication  *app,
+                         GFile        **files,
+                         int            n_files,
+                         const char    *hint)
+{
+  PtyxisApplication *self = (PtyxisApplication *)app;
+  g_autoptr(PtyxisProfile) profile = NULL;
+  PtyxisWindow *window = NULL;
+
+  g_assert (PTYXIS_IS_APPLICATION (self));
+  g_assert (PTYXIS_IS_CLIENT (self->client));
+  g_assert (files != NULL);
+
+  if (n_files == 0)
+    return;
+
+  /* We want to restore the session so the user doesn't lose it. Only in that
+   * case do we want to add a tab to an existing window because otherwise we
+   * might added it to a window on another workspace we cannot bring-to-front.
+   */
+  if (ptyxis_application_restore (self))
+    {
+      for (const GList *windows = gtk_application_get_windows (GTK_APPLICATION (app));
+           windows != NULL;
+           windows = windows->next)
+        {
+          if (PTYXIS_IS_WINDOW (windows->data))
+            {
+              window = windows->data;
+              break;
+            }
+        }
+    }
+
+  if (window == NULL)
+    window = ptyxis_window_new_empty ();
+
+  profile = ptyxis_application_dup_default_profile (self);
+
+  for (guint i = 0; i < n_files; i++)
+    {
+      PtyxisTab *tab = ptyxis_tab_new (profile);
+      PtyxisTerminal *terminal = ptyxis_tab_get_terminal (tab);
+      g_autofree char *uri = g_file_get_uri (files[i]);
+
+      ptyxis_application_apply_default_size (self, terminal);
+      ptyxis_tab_set_initial_working_directory_uri (tab, uri);
+      ptyxis_window_add_tab (window, tab);
+
+      if (i == 0)
+        ptyxis_window_set_active_tab (window, tab);
+    }
+
+  gtk_window_present (GTK_WINDOW (window));
+}
+
 static void
 ptyxis_application_activate (GApplication *app)
 {
@@ -170,6 +228,11 @@ ptyxis_application_activate (GApplication *app)
   if (!ptyxis_application_restore (self))
     {
       PtyxisWindow *window = ptyxis_window_new ();
+      PtyxisTab *tab = ptyxis_window_get_active_tab (window);
+
+      ptyxis_tab_set_title_prefix (tab, self->next_title_prefix);
+      g_clear_pointer (&self->next_title_prefix, g_free);
+
       gtk_window_present (GTK_WINDOW (window));
     }
 }
@@ -196,6 +259,25 @@ get_current_window (PtyxisApplication *self)
   return NULL;
 }
 
+static void
+ptyxis_application_apply_default_size (PtyxisApplication *self,
+                                       PtyxisTerminal    *terminal)
+{
+  guint columns;
+  guint rows;
+
+  g_assert (PTYXIS_IS_APPLICATION (self));
+  g_assert (PTYXIS_IS_TERMINAL (terminal));
+  g_assert (PTYXIS_IS_SETTINGS (self->settings));
+
+  if (ptyxis_settings_get_restore_window_size (self->settings))
+    ptyxis_settings_get_window_size (self->settings, &columns, &rows);
+  else
+    ptyxis_settings_get_default_size (self->settings, &columns, &rows);
+
+  vte_terminal_set_size (VTE_TERMINAL (terminal), columns, rows);
+}
+
 static int
 ptyxis_application_command_line (GApplication            *app,
                                  GApplicationCommandLine *cmdline)
@@ -205,6 +287,7 @@ ptyxis_application_command_line (GApplication            *app,
   g_autofree char *working_directory = NULL;
   g_autofree char *command = NULL;
   g_autofree char *cwd_uri = NULL;
+  g_autofree char *title = NULL;
   g_auto(GStrv) argv = NULL;
   GVariantDict *dict;
   const char *cwd;
@@ -239,6 +322,9 @@ ptyxis_application_command_line (GApplication            *app,
   if (!g_variant_dict_lookup (dict, "new-window", "b", &new_window))
     new_window = FALSE;
 
+  if (!g_variant_dict_lookup (dict, "title", "s", &title))
+    title = NULL;
+
   if (new_tab && new_window)
     {
       g_application_command_line_printerr (cmdline,
@@ -301,7 +387,10 @@ ptyxis_application_command_line (GApplication            *app,
 
           if (window == NULL)
             window = ptyxis_window_new_empty ();
+
           tab = ptyxis_window_add_tab_for_command (window, NULL, (const char * const *)argv, cwd_uri);
+
+          ptyxis_tab_set_title_prefix (tab, title);
           ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
           ptyxis_window_set_active_tab (window, tab);
           gtk_window_present (GTK_WINDOW (window));
@@ -314,7 +403,10 @@ ptyxis_application_command_line (GApplication            *app,
 
           if (window == NULL || new_window)
             window = ptyxis_window_new_empty ();
+
           tab = ptyxis_window_add_tab_for_command (window, profile, (const char * const *)argv, cwd_uri);
+
+          ptyxis_tab_set_title_prefix (tab, title);
           ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
           ptyxis_window_set_active_tab (window, tab);
           gtk_window_present (GTK_WINDOW (window));
@@ -326,15 +418,23 @@ ptyxis_application_command_line (GApplication            *app,
 
           window = ptyxis_window_new_empty ();
           tab = ptyxis_window_add_tab_for_command (window, NULL, (const char * const *)argv, cwd_uri);
+
+          ptyxis_tab_set_title_prefix (tab, title);
           ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
           ptyxis_window_set_active_tab (window, tab);
+
           gtk_window_present (GTK_WINDOW (window));
         }
       else
         {
           PtyxisWindow *window;
+          PtyxisTab *tab;
 
           window = ptyxis_window_new_for_command (NULL, (const char * const *)argv, cwd_uri);
+          tab = ptyxis_window_get_active_tab (window);
+
+          ptyxis_tab_set_title_prefix (tab, title);
+
           gtk_application_add_window (GTK_APPLICATION (self), GTK_WINDOW (window));
           gtk_window_present (GTK_WINDOW (window));
         }
@@ -345,14 +445,19 @@ ptyxis_application_command_line (GApplication            *app,
       g_autoptr(PtyxisProfile) profile = ptyxis_application_dup_default_profile (self);
       PtyxisWindow *window = get_current_window (self);
       PtyxisTab *tab = ptyxis_tab_new (profile);
+      PtyxisTerminal *terminal = ptyxis_tab_get_terminal (tab);
 
       if (window == NULL || new_window)
-        window = ptyxis_window_new_empty ();
+        {
+          window = ptyxis_window_new_empty ();
+          ptyxis_application_apply_default_size (self, terminal);
+        }
 
       ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
-
+      ptyxis_tab_set_title_prefix (tab, title);
       ptyxis_window_add_tab (window, tab);
       ptyxis_window_set_active_tab (window, tab);
+
       gtk_window_present (GTK_WINDOW (window));
     }
   else if (new_tab_with_profile)
@@ -360,14 +465,19 @@ ptyxis_application_command_line (GApplication            *app,
       g_autoptr(PtyxisProfile) profile = ptyxis_application_dup_profile (self, new_tab_with_profile);
       PtyxisWindow *window = get_current_window (self);
       PtyxisTab *tab = ptyxis_tab_new (profile);
+      PtyxisTerminal *terminal = ptyxis_tab_get_terminal (tab);
 
       if (window == NULL || new_window)
-        window = ptyxis_window_new_empty ();
+        {
+          window = ptyxis_window_new_empty ();
+          ptyxis_application_apply_default_size (self, terminal);
+        }
 
       ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
-
+      ptyxis_tab_set_title_prefix (tab, title);
       ptyxis_window_add_tab (window, tab);
       ptyxis_window_set_active_tab (window, tab);
+
       gtk_window_present (GTK_WINDOW (window));
     }
   else if (g_variant_dict_contains (dict, "new-window"))
@@ -375,23 +485,16 @@ ptyxis_application_command_line (GApplication            *app,
       g_autoptr(PtyxisProfile) profile = ptyxis_application_dup_default_profile (self);
       PtyxisWindow *window = get_current_window (self);
       PtyxisTab *tab = ptyxis_tab_new (profile);
+      PtyxisTerminal *terminal = ptyxis_tab_get_terminal (tab);
 
       if (window == NULL || !did_restore)
         {
-          PtyxisTerminal *terminal = ptyxis_tab_get_terminal (tab);
-          guint columns, rows;
-
           window = ptyxis_window_new_empty ();
-
-          if (ptyxis_settings_get_restore_window_size (self->settings))
-            ptyxis_settings_get_window_size (self->settings, &columns, &rows);
-          else
-            ptyxis_settings_get_default_size (self->settings, &columns, &rows);
-
-          vte_terminal_set_size (VTE_TERMINAL (terminal), columns, rows);
+          ptyxis_application_apply_default_size (self, terminal);
         }
 
       ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
+      ptyxis_tab_set_title_prefix (tab, title);
       ptyxis_window_add_tab (window, tab);
       ptyxis_window_set_active_tab (window, tab);
 
@@ -399,6 +502,8 @@ ptyxis_application_command_line (GApplication            *app,
     }
   else
     {
+      g_set_str (&self->next_title_prefix, title);
+
       g_application_activate (G_APPLICATION (self));
     }
 
@@ -568,34 +673,8 @@ ptyxis_application_client_closed_cb (PtyxisApplication *self,
   g_assert (PTYXIS_IS_APPLICATION (self));
   g_assert (PTYXIS_IS_CLIENT (client));
 
-  /* We can reach this in two cases. The first is the case where the
-   * desktop session is exiting and our `ptyxis-agent` got nuked before
-   * we did.
-   *
-   * The second is if there was a crash by the client. For that, we
-   * should get a crash report _anyway_ so just exit cleanly here.
-   */
-
-  exit (EXIT_SUCCESS);
-}
-
-static void
-ptyxis_application_client_process_exited_cb (PtyxisApplication *self,
-                                             const char        *process_object_path,
-                                             int                exit_code,
-                                             PtyxisClient      *client)
-{
-  g_assert (PTYXIS_IS_APPLICATION (self));
-  g_assert (PTYXIS_IS_CLIENT (client));
-
-  /* Store information about how the process exited. We will use this if
-   * we race to figure out how the process exited otherwise.  It is
-   * removed from the hash table when doing so (or when we get the real
-   * value outside of a race condition).
-   */
-  g_hash_table_insert (self->exited,
-                       g_strdup (process_object_path),
-                       GINT_TO_POINTER (exit_code));
+  /* We want a stack trace generated because this should not happen */
+  g_error ("ptyxis-agent exited unexpectedly. exiting too.");
 }
 
 static void
@@ -616,7 +695,6 @@ ptyxis_application_startup (GApplication *application)
   g_application_set_default (application);
   g_application_set_resource_base_path (G_APPLICATION (self), "/org/gnome/Ptyxis");
 
-  self->exited = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   self->profiles = g_list_store_new (PTYXIS_TYPE_PROFILE);
   self->settings = ptyxis_settings_new ();
   self->shortcuts = ptyxis_shortcuts_new (NULL);
@@ -664,12 +742,6 @@ ptyxis_application_startup (GApplication *application)
                            self,
                            G_CONNECT_SWAPPED);
 
-  g_signal_connect_object (self->client,
-                           "process-exited",
-                           G_CALLBACK (ptyxis_application_client_process_exited_cb),
-                           self,
-                           G_CONNECT_SWAPPED);
-
   self->profile_menu = ptyxis_profile_menu_new (self->settings);
 
   filter = gtk_custom_filter_new (filter_session_container, NULL, NULL);
@@ -750,7 +822,7 @@ ptyxis_application_shutdown (GApplication *application)
   g_clear_object (&self->shortcuts);
   g_clear_object (&self->settings);
   g_clear_object (&self->client);
-  g_clear_pointer (&self->exited, g_hash_table_unref);
+  g_clear_pointer (&self->next_title_prefix, g_free);
 
   g_clear_pointer (&self->system_font_name, g_free);
 }
@@ -798,6 +870,7 @@ ptyxis_application_class_init (PtyxisApplicationClass *klass)
   app_class->startup = ptyxis_application_startup;
   app_class->shutdown = ptyxis_application_shutdown;
   app_class->command_line = ptyxis_application_command_line;
+  app_class->open = ptyxis_application_open;
 
   properties[PROP_DEFAULT_PROFILE] =
     g_param_spec_object ("default-profile", NULL, NULL,
@@ -849,6 +922,8 @@ ptyxis_application_init (PtyxisApplication *self)
     { "tab", 0, 0, G_OPTION_ARG_NONE, NULL, N_("New terminal tab in active window") },
     { "tab-with-profile", 0, 0, G_OPTION_ARG_STRING, NULL, N_("New terminal tab in active window using PROFILE"), N_("PROFILE") },
 
+    { "title", 0, 0, G_OPTION_ARG_STRING, NULL, N_("Set title for new tab") },
+
     { NULL }
   };
 
@@ -1098,7 +1173,7 @@ ptyxis_application_about (GSimpleAction *action,
                          "application-icon", PACKAGE_ICON_NAME,
                          "application-name", ptyxis_app_name (),
                          "artists", artists,
-                         "copyright", "© 2023-2024 Christian Hergert",
+                         "copyright", "© 2023-2024 Christian Hergert, et al.",
                          "debug-info", debug_info,
                          "developer-name", "Christian Hergert",
                          "developers", developers,
@@ -1640,9 +1715,6 @@ ptyxis_application_process_exited_cb (PtyxisIpcProcess *process,
   g_assert (PTYXIS_IPC_IS_PROCESS (process));
   g_assert (G_IS_TASK (task));
 
-  g_hash_table_remove (PTYXIS_APPLICATION_DEFAULT->exited,
-                       g_dbus_proxy_get_object_path (G_DBUS_PROXY (process)));
-
   wait_complete (task, exit_status, 0, NULL);
 }
 
@@ -1654,9 +1726,6 @@ ptyxis_application_process_signaled_cb (PtyxisIpcProcess *process,
   g_assert (PTYXIS_IPC_IS_PROCESS (process));
   g_assert (G_IS_TASK (task));
 
-  g_hash_table_remove (PTYXIS_APPLICATION_DEFAULT->exited,
-                       g_dbus_proxy_get_object_path (G_DBUS_PROXY (process)));
-
   wait_complete (task, 0, term_sig, NULL);
 }
 
@@ -1665,7 +1734,6 @@ ptyxis_application_has_foreground_process_cb (GObject      *object,
                                               GAsyncResult *result,
                                               gpointer      user_data)
 {
-  PtyxisApplication *self = PTYXIS_APPLICATION_DEFAULT;
   PtyxisIpcProcess *process = (PtyxisIpcProcess *)object;
   g_autoptr(GError) error = NULL;
   g_autoptr(GTask) task = user_data;
@@ -1675,28 +1743,7 @@ ptyxis_application_has_foreground_process_cb (GObject      *object,
   g_assert (G_IS_TASK (task));
 
   if (!ptyxis_ipc_process_call_has_foreground_process_finish (process, NULL, NULL, NULL, NULL, NULL, result, &error))
-    {
-      const char *object_path = g_dbus_proxy_get_object_path (G_DBUS_PROXY (process));
-      gpointer value;
-      int exit_status = 0;
-      int term_sig = 0;
-
-      if (g_hash_table_lookup_extended (self->exited, object_path, NULL, &value))
-        {
-          int exit_code = GPOINTER_TO_INT (value);
-
-          if (WIFEXITED (exit_code))
-            exit_status = WEXITSTATUS (exit_code);
-          else
-            term_sig = WTERMSIG (exit_code);
-
-          g_clear_error (&error);
-
-          g_hash_table_remove (self->exited, object_path);
-        }
-
-      wait_complete (task, exit_status, term_sig, g_steal_pointer (&error));
-    }
+    wait_complete (task, 0, 0, g_steal_pointer (&error));
 }
 
 void
