@@ -72,6 +72,8 @@ struct _PtyxisTerminal
 
 enum {
   PROP_0,
+  PROP_CURRENT_CONTAINER_NAME,
+  PROP_CURRENT_CONTAINER_RUNTIME,
   PROP_PALETTE,
   PROP_SHORTCUTS,
   N_PROPS
@@ -80,6 +82,8 @@ enum {
 enum {
   MATCH_CLICKED,
   GRID_SIZE_CHANGED,
+  SHELL_PRECMD,
+  SHELL_PREEXEC,
   N_SIGNALS
 };
 
@@ -100,6 +104,7 @@ ptyxis_terminal_update_colors (PtyxisTerminal *self)
   const PtyxisPaletteFace *face;
   AdwStyleManager *style_manager;
   gboolean dark;
+  GdkRGBA fallback;
 
   g_assert (PTYXIS_IS_TERMINAL (self));
 
@@ -126,8 +131,13 @@ ptyxis_terminal_update_colors (PtyxisTerminal *self)
     }
   else
     {
-      vte_terminal_set_color_cursor (VTE_TERMINAL (self), NULL);
-      vte_terminal_set_color_cursor_foreground (VTE_TERMINAL (self), NULL);
+      if (dark)
+        fallback = (GdkRGBA){1.0, 1.0, 1.0, 1.0};
+      else
+        fallback = (GdkRGBA){0.0, 0.0, 0.0, 1.0};
+
+      vte_terminal_set_color_cursor (VTE_TERMINAL (self), &fallback);
+      vte_terminal_set_color_cursor_foreground (VTE_TERMINAL (self), &face->background);
     }
 }
 
@@ -1044,6 +1054,30 @@ ptyxis_terminal_shortcuts_notify_cb (PtyxisTerminal  *self,
   ptyxis_shortcuts_update_menu (shortcuts, self->terminal_menu);
 }
 
+static void
+ptyxis_terminal_emit_shell_precmd (PtyxisTerminal *self)
+{
+  g_assert (PTYXIS_IS_TERMINAL (self));
+
+  g_signal_emit (self, signals[SHELL_PRECMD], 0);
+}
+
+static void
+ptyxis_terminal_emit_shell_preexec (PtyxisTerminal *self)
+{
+  g_assert (PTYXIS_IS_TERMINAL (self));
+
+  g_signal_emit (self, signals[SHELL_PREEXEC], 0);
+}
+
+static void
+notify_property_changed (PtyxisTerminal *self,
+                         const char     *termprop,
+                         GParamSpec     *property)
+{
+  g_object_notify_by_pspec (G_OBJECT (self), property);
+}
+
 static void
 ptyxis_terminal_constructed (GObject *object)
 {
@@ -1091,6 +1125,20 @@ ptyxis_terminal_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_CURRENT_CONTAINER_NAME:
+      g_value_set_string (value,
+                          vte_terminal_get_termprop_string_by_id (VTE_TERMINAL (self),
+                                                                  VTE_PROPERTY_ID_CONTAINER_NAME,
+                                                                  NULL));
+      break;
+
+    case PROP_CURRENT_CONTAINER_RUNTIME:
+      g_value_set_string (value,
+                          vte_terminal_get_termprop_string_by_id (VTE_TERMINAL (self),
+                                                                  VTE_PROPERTY_ID_CONTAINER_RUNTIME,
+                                                                  NULL));
+      break;
+
     case PROP_PALETTE:
       g_value_set_object (value, ptyxis_terminal_get_palette (self));
       break;
@@ -1142,6 +1190,18 @@ ptyxis_terminal_class_init (PtyxisTerminalClass *klass)
   terminal_class->selection_changed = ptyxis_terminal_selection_changed;
   terminal_class->setup_context_menu = ptyxis_terminal_setup_context_menu;
 
+  properties[PROP_CURRENT_CONTAINER_NAME] =
+    g_param_spec_string ("current-container-name", NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_CURRENT_CONTAINER_RUNTIME] =
+    g_param_spec_string ("current-container-runtime", NULL, NULL,
+                         NULL,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+
   properties[PROP_PALETTE] =
     g_param_spec_object ("palette", NULL, NULL,
                          PTYXIS_TYPE_PALETTE,
@@ -1181,6 +1241,24 @@ ptyxis_terminal_class_init (PtyxisTerminalClass *klass)
                   GDK_TYPE_MODIFIER_TYPE,
                   G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
 
+  signals[SHELL_PRECMD] =
+    g_signal_new ("shell-precmd",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL,
+                  NULL,
+                  G_TYPE_NONE, 0);
+
+  signals[SHELL_PREEXEC] =
+    g_signal_new ("shell-preexec",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL,
+                  NULL,
+                  G_TYPE_NONE, 0);
+
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Ptyxis/ptyxis-terminal.ui");
 
   gtk_widget_class_bind_template_child (widget_class, PtyxisTerminal, drop_highlight);
@@ -1264,11 +1342,22 @@ ptyxis_terminal_init (PtyxisTerminal *self)
                                       GDK_ACTION_MOVE));
   gtk_drop_target_async_set_formats (self->drop_target, formats);
 
-  g_signal_connect_object (gtk_widget_get_clipboard (GTK_WIDGET (self)),
-                           "changed",
-                           G_CALLBACK (ptyxis_terminal_update_clipboard_actions),
-                           self,
-                           G_CONNECT_SWAPPED);
+  g_signal_connect (self,
+                    "termprop-changed::" VTE_TERMPROP_SHELL_PRECMD,
+                    G_CALLBACK (ptyxis_terminal_emit_shell_precmd),
+                    NULL);
+  g_signal_connect (self,
+                    "termprop-changed::" VTE_TERMPROP_SHELL_PREEXEC,
+                    G_CALLBACK (ptyxis_terminal_emit_shell_preexec),
+                    NULL);
+  g_signal_connect (self,
+                    "termprop-changed::" VTE_TERMPROP_CONTAINER_NAME,
+                    G_CALLBACK (notify_property_changed),
+                    properties[PROP_CURRENT_CONTAINER_NAME]);
+  g_signal_connect (self,
+                    "termprop-changed::" VTE_TERMPROP_CONTAINER_RUNTIME,
+                    G_CALLBACK (notify_property_changed),
+                    properties[PROP_CURRENT_CONTAINER_RUNTIME]);
 
   ptyxis_terminal_update_clipboard_actions (self);
 }
@@ -1293,3 +1382,59 @@ ptyxis_terminal_set_palette (PtyxisTerminal *self,
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PALETTE]);
     }
 }
+
+const char *
+ptyxis_terminal_get_current_container_name (PtyxisTerminal *self)
+{
+  gsize len;
+
+  g_return_val_if_fail (PTYXIS_IS_TERMINAL (self), NULL);
+
+  return vte_terminal_get_termprop_string (VTE_TERMINAL (self),
+                                           VTE_TERMPROP_CONTAINER_NAME,
+                                           &len);
+}
+
+const char *
+ptyxis_terminal_get_current_container_runtime (PtyxisTerminal *self)
+{
+  gsize len;
+
+  g_return_val_if_fail (PTYXIS_IS_TERMINAL (self), NULL);
+
+  return vte_terminal_get_termprop_string (VTE_TERMINAL (self),
+                                           VTE_TERMPROP_CONTAINER_RUNTIME,
+                                           &len);
+}
+
+static char *
+ptyxis_terminal_dup_termprop_uri_by_id (PtyxisTerminal *self,
+                                        guint           property)
+{
+  g_autoptr(GUri) uri = NULL;
+
+  g_assert (PTYXIS_IS_TERMINAL (self));
+
+  if ((uri = vte_terminal_ref_termprop_uri_by_id (VTE_TERMINAL (self), property)))
+    return g_uri_to_string (uri);
+
+  return NULL;
+}
+
+char *
+ptyxis_terminal_dup_current_directory_uri (PtyxisTerminal *self)
+{
+  g_return_val_if_fail (PTYXIS_IS_TERMINAL (self), NULL);
+
+  return ptyxis_terminal_dup_termprop_uri_by_id (self, VTE_PROPERTY_ID_CURRENT_DIRECTORY_URI);
+}
+
+char *
+ptyxis_terminal_dup_current_file_uri (PtyxisTerminal *self)
+{
+  g_autoptr(GUri) uri = NULL;
+
+  g_return_val_if_fail (PTYXIS_IS_TERMINAL (self), NULL);
+
+  return ptyxis_terminal_dup_termprop_uri_by_id (self, VTE_PROPERTY_ID_CURRENT_FILE_URI);
+}
