@@ -253,14 +253,14 @@ gboolean
 ptyxis_is_shell (const char *arg0)
 {
   static const char * const builtin_shells[] = {
-    "sh", "/bin/sh", "/usr/bin/sh",
-    "bash", "/bin/bash", "/usr/bin/bash",
-    "dash", "/bin/dash", "/usr/bin/dash",
-    "zsh", "/bin/zsh", "/usr/bin/zsh",
-    "fish", "/bin/fish", "/usr/bin/fish",
-    "tcsh", "/bin/tcsh", "/usr/bin/tcsh",
-    "csh", "/bin/csh", "/usr/bin/csh",
-    "tmux", "/bin/tmux", "/usr/bin/tmux",
+    "/bin/sh", "/usr/bin/sh",
+    "/bin/bash", "/usr/bin/bash",
+    "/bin/dash", "/usr/bin/dash",
+    "/bin/zsh", "/usr/bin/zsh",
+    "/bin/fish", "/usr/bin/fish",
+    "/bin/tcsh", "/usr/bin/tcsh",
+    "/bin/csh", "/usr/bin/csh",
+    "/bin/tmux", "/usr/bin/tmux",
   };
   const char *etc_shells_path = "/etc/shells";
   g_autofree char *etc_shells = NULL;
@@ -304,9 +304,25 @@ ptyxis_parse_shells (const char *etc_shells)
 const char *
 ptyxis_app_name (void)
 {
-#if APP_IS_GENERIC
+#if APP_IS_BUILDER
+  /* translators: Builder Terminal means this is a terminal bundled with GNOME Builder */
+  return _("Builder Terminal");
+#elif APP_IS_GENERIC
   return _("Terminal");
 #else
   return _("Ptyxis");
 #endif
 }
+
+GVariant *
+ptyxis_variant_new_toast (const char *title,
+                          guint       timeout)
+{
+  GVariantBuilder b;
+
+  g_variant_builder_init (&b, G_VARIANT_TYPE ("a{sv}"));
+  g_variant_builder_add (&b, "{sv}", "title", g_variant_new_string (title));
+  g_variant_builder_add (&b, "{sv}", "timeout", g_variant_new_uint32 (timeout));
+
+  return g_variant_builder_end (&b);
+}
