@@ -127,6 +127,10 @@ ptyxis_window_close_page_dialog_cb (GObject      *object,
 
   ptyxis_parking_lot_push (self->parking_lot, tab);
   adw_tab_view_close_page_finish (self->tab_view, page, TRUE);
+
+  /* Resize if we are going from 2->1 tabs */
+  if (adw_tab_view_get_n_pages (self->tab_view) == 1)
+    gtk_window_set_default_size (GTK_WINDOW (self), -1, -1);
 }
 
 static gboolean
@@ -148,6 +152,7 @@ ptyxis_window_close_page_cb (PtyxisWindow *self,
   if (!ptyxis_tab_is_running (tab, NULL))
     {
       ptyxis_parking_lot_push (self->parking_lot, tab);
+      gtk_window_set_default_size (GTK_WINDOW (self), -1, -1);
       return GDK_EVENT_PROPAGATE;
     }
 
@@ -155,10 +160,10 @@ ptyxis_window_close_page_cb (PtyxisWindow *self,
   g_ptr_array_add (tabs, g_object_ref (tab));
 
   _ptyxis_close_dialog_run_async (GTK_WINDOW (self),
-                                   tabs,
-                                   NULL,
-                                   ptyxis_window_close_page_dialog_cb,
-                                   g_object_ref (tab));
+                                  tabs,
+                                  NULL,
+                                  ptyxis_window_close_page_dialog_cb,
+                                  g_object_ref (tab));
 
   return GDK_EVENT_STOP;
 }
@@ -400,9 +405,9 @@ ptyxis_window_apply_current_settings (PtyxisWindow *self,
   if ((active_tab = ptyxis_window_get_active_tab (self)))
     {
       PtyxisTerminal *terminal = ptyxis_tab_get_terminal (active_tab);
-      const char *current_directory_uri = ptyxis_tab_get_current_directory_uri (active_tab);
-      const char *current_container_name = vte_terminal_get_current_container_name (VTE_TERMINAL (terminal));
-      const char *current_container_runtime = vte_terminal_get_current_container_runtime (VTE_TERMINAL (terminal));
+      g_autofree char *current_directory_uri = ptyxis_tab_dup_current_directory_uri (active_tab);
+      const char *current_container_name = ptyxis_terminal_get_current_container_name (terminal);
+      const char *current_container_runtime = ptyxis_terminal_get_current_container_runtime (terminal);
       PtyxisZoomLevel zoom = ptyxis_tab_get_zoom (active_tab);
       g_autoptr(PtyxisIpcContainer) current_container = NULL;
 
@@ -514,8 +519,6 @@ ptyxis_window_new_window_action (GtkWidget  *widget,
   const char *profile_uuid = "";
   const char *container_id = "";
   PtyxisTab *tab;
-  GdkToplevel *toplevel;
-  GdkToplevelState state;
   guint columns, rows;
 
   g_assert (PTYXIS_IS_WINDOW (self));
@@ -538,16 +541,12 @@ ptyxis_window_new_window_action (GtkWidget  *widget,
         ptyxis_tab_set_container (tab, container);
     }
 
-  toplevel = GDK_TOPLEVEL (gtk_native_get_surface (GTK_NATIVE (self)));
-  state = gdk_toplevel_get_state (toplevel);
-
   /* If the current window is maximized, don't maximize this window as
    * it's most likely they're just doing a temporary thing or would like
    * to move the window elsewhere.
    */
-  if (!!(state & (GDK_TOPLEVEL_STATE_MAXIMIZED |
-                  GDK_TOPLEVEL_STATE_FULLSCREEN |
-                  GDK_TOPLEVEL_STATE_TILED)))
+  if (gtk_window_is_maximized (GTK_WINDOW (self)) ||
+      gtk_window_is_fullscreen (GTK_WINDOW (self)))
     {
       PtyxisTerminal *terminal = ptyxis_tab_get_terminal (tab);
       ptyxis_settings_get_default_size (settings, &columns, &rows);
@@ -1840,6 +1839,10 @@ ptyxis_window_add_tab (PtyxisWindow *self,
 
   adw_tab_view_insert (self->tab_view, GTK_WIDGET (tab), position);
 
+  /* Resize if we are going from 1->2 tabs */
+  if (adw_tab_view_get_n_pages (self->tab_view) == 2)
+    gtk_window_set_default_size (GTK_WINDOW (self), -1, -1);
+
   gtk_widget_grab_focus (GTK_WIDGET (tab));
 }
 
@@ -1974,7 +1977,6 @@ ptyxis_window_focus_tab_by_uuid (PtyxisWindow *self,
       if (0 == g_strcmp0 (uuid, ptyxis_tab_get_uuid (tab)))
         {
           ptyxis_window_set_active_tab (self, tab);
-          gtk_window_present (GTK_WINDOW (self));
           return TRUE;
         }
     }
